function DeepCopy(orig)
	local orig_type = type(orig)
	local copy

	if orig_type == "table" then
		copy = {}

		for orig_key, orig_value in next, orig, nil do
			copy[DeepCopy(orig_key)] = DeepCopy(orig_value)
		end

		setmetatable(copy, DeepCopy(getmetatable(orig)))
	else
		copy = orig
	end

	return copy
end

local MoveDefs = DeepCopy(DEFS.moveDefs)
local UnitDefs = DeepCopy(DEFS.unitDefs)
local WeaponDefs = DeepCopy(DEFS.weaponDefs)

for name, mDef in pairs(MoveDefs) do
	if mDef.footprintx and mDef.footprintz then
		mDef.footprintx = mDef.footprintx - 1
		mDef.footprintz = mDef.footprintx - 1
	end

	mDef.heatmapping = false
	mDef.allowrawmovement = true
	mDef.avoidmobilesonpath = false
	mDef.flowmapping = false
end

for name, uDef in pairs(UnitDefs) do
	if uDef.footprintx and uDef.footprintx > 20 then
		uDef.footprintx = 20
		uDef.footprintz = 20
	end

	if uDef.mass and uDef.mass > 20000 then
		uDef.mass = 20000
	end

	uDef.mintransportmass = 0
	uDef.mintransportsize = 0
	uDef.transportbyenemy = false
	uDef.transportmass = 100000
	uDef.isfireplatform = true
	uDef.transportsize = 100
end

for name, wDef in pairs(WeaponDefs) do
	wDef.avoidfeature = false
	wDef.allownonblockingaim = false
end

DEFS.moveDefs = MoveDefs
DEFS.unitDefs = UnitDefs
DEFS.weaponDefs = WeaponDefs