function gadget:GetInfo()
	return {
		name = "Map Setup",
		desc = "",
		author = "[MOL]",
		date = "05.08.2021",
		license = "",
		layer = 1,
		version = "1.1",
		enabled = true
	}
end

VFS.Include("luarules/configs/metalSpots.lua")
VFS.Include("luarules/configs/geoVents.lua")
mex = VFS.Include("luarules/gadgets/include/mex_utilities.lua")
vent = VFS.Include("luarules/gadgets/include/vent_utilities.lua")
local map_metal = tonumber(Spring.GetMapOptions().map_metal)
local mapX = Game.mapSizeX
local mapZ = Game.mapSizeZ
local SpotsOffset = 160
local edgeOffset = 170

local mexSpots = {
	{
		m = map_metal,
		sx = -SpotsOffset,
		sz = -SpotsOffset,
	},
	{
		m = map_metal,
		sx = -SpotsOffset,
		sz = SpotsOffset
	},
	{
		m = map_metal,
		sx = SpotsOffset,
		sz = SpotsOffset
	},
	{
		m = map_metal,
		sx = SpotsOffset,
		sz = -SpotsOffset
	}
}

local geoVents = {
	{
		sx = -SpotsOffset / 1.7,
		sz = 0.0,
	},
	{
		sx = SpotsOffset / 1.7,
		sz = 0.0,
	}
}

if (gadgetHandler:IsSyncedCode()) then
	local PlayerGeoVents = {}
	local PlayerMetalSpots = {}
	local prevPlayerGeoVents = {}
	local prevPlayerMetalSpots = {}

	function DeepCopy(orig)
		local orig_type = type(orig)
		local copy

		if orig_type == "table" then
			copy = {}

			for orig_key, orig_value in next, orig, nil do
				copy[DeepCopy(orig_key)] = DeepCopy(orig_value)
			end

			setmetatable(copy, DeepCopy(getmetatable(orig)))
		else
			copy = orig
		end

		return copy
	end

	function gadget:Initialize()
		mex.clearAll()
		mex.set(constMexSpots)
		vent.set(constVentsSpots)
	end

	function gadget:GameStart()
		local gaiaTeamID = Spring.GetGaiaTeamID()
		local teamList = Spring.GetTeamList()

		for i = 1, #teamList do
			local teamID = teamList[i]

			if teamID ~= gaiaTeamID and PlayerMetalSpots[teamID] == nil then
				x0, _, z0 = Spring.GetTeamStartPosition(teamID)

				for _, v in ipairs(mexSpots) do
					v.x = v.sx + x0
					v.z = v.sz + z0
				end

				for _, v in ipairs(geoVents) do
					v.x = v.sx + x0
					v.z = v.sz + z0
				end

				PlayerMetalSpots[teamID] = DeepCopy(mexSpots)
				PlayerGeoVents[teamID] = DeepCopy(geoVents)
				mex.set(PlayerMetalSpots[teamID])
				vent.set(PlayerGeoVents[teamID])
			end
		end

		_G.mexSpots = PlayerMetalSpots
		_G.geoVents = PlayerGeoVents
		SendToUnsynced("SpotsUpdate")
	end

	function gadget:AllowStartPosition(playerID, teamID, readyState, x, y, z, rx, ry, rz)
		if not x or not z or x < edgeOffset or x > mapX - edgeOffset or z < edgeOffset or z > mapZ - edgeOffset then return false end

		for _, v in ipairs(mexSpots) do
			v.x = v.sx + x
			v.z = v.sz + z
		end

		for _, v in ipairs(geoVents) do
			v.x = v.sx + x
			v.z = v.sz + z
		end

		prevPlayerGeoVents[playerID] = PlayerGeoVents[playerID]
		prevPlayerMetalSpots[playerID] = PlayerMetalSpots[playerID]

		if prevPlayerMetalSpots[playerID] then
			mex.clear(prevPlayerMetalSpots[playerID])
		end

		if prevPlayerGeoVents[playerID] then
			vent.clear(prevPlayerGeoVents[playerID])
		end

		PlayerMetalSpots[playerID] = DeepCopy(mexSpots)
		PlayerGeoVents[playerID] = DeepCopy(geoVents)
		mex.set(PlayerMetalSpots[playerID])
		vent.set(PlayerGeoVents[playerID])
		_G.mexSpots = PlayerMetalSpots
		_G.geoVents = PlayerGeoVents
		SendToUnsynced("SpotsUpdate")
		mex.set(constMexSpots)

		return true
	end
else
	local displayList = 0
	local MexWidth = 130
	local MexHeight = 130
	local halfMexWidth = MexWidth / 2
	local halfMexHeight = MexHeight / 2
	local geoVentWidth = 90
	local geoVentHeight = 90
	local halfGeoVentWidth = geoVentWidth / 2
	local halfGeoVentHeight = geoVentHeight / 2
	local mexPlayersSpots = {}
	local ventsPlayersSpots = {}

	function gadget:Initialize()
		gadgetHandler:AddSyncAction("SpotsUpdate", SpotsUpdate)
		SpotsUpdate()
	end

	function gadget:ShutDown()
		gadgetHandler:RemoveSyncAction("SpotsUpdate")

		if displayList then
			gl.DeleteList(displayList)
		end
	end

	function SpotsUpdate()
		mexPlayersSpots = SYNCED.mexSpots or {}
		ventsPlayersSpots = SYNCED.geoVents or {}

		if displayList then
			gl.DeleteList(displayList)
		end

		displayList = gl.CreateList(drawPatches)
	end

	function drawPatches()
		gl.MatrixMode(GL.TEXTURE)
		gl.PolygonOffset(-25, -2)
		gl.Culling(GL.BACK)
		gl.DepthTest(true)
		gl.Color(1, 1, 1, 1)
		gl.Texture(":a:luaui/images/mex.png")

		for playerID, mexSpot in pairs(mexPlayersSpots) do
			for i = 1, #mexSpot do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(mexSpot[i].x - halfMexWidth, mexSpot[i].z - halfMexHeight, mexSpot[i].x + halfMexWidth, mexSpot[i].z + halfMexHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		if constMexSpots then
			for i = 1, #constMexSpots do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(constMexSpots[i].x - halfMexWidth, constMexSpots[i].z - halfMexHeight, constMexSpots[i].x + halfMexWidth, constMexSpots[i].z + halfMexHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		gl.Texture(":a:luaui/images/geo.png")

		for playerID, geoVent in pairs(ventsPlayersSpots) do
			for i = 1, #geoVent do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(geoVent[i].x - halfGeoVentWidth, geoVent[i].z - halfGeoVentHeight, geoVent[i].x + halfGeoVentWidth, geoVent[i].z + halfGeoVentHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		if constVentsSpots then
			for i = 1, #constVentsSpots do
				gl.PushMatrix()
				gl.Translate(0.5, 0.5, 0)
				gl.DrawGroundQuad(constVentsSpots[i].x - halfGeoVentWidth, constVentsSpots[i].z - halfGeoVentHeight, constVentsSpots[i].x + halfGeoVentWidth, constVentsSpots[i].z + halfGeoVentHeight, false, -0.5, -0.5, 0.5, 0.5)
				gl.PopMatrix()
			end
		end

		gl.Texture(false)
		gl.DepthTest(false)
		gl.Culling(false)
		gl.PolygonOffset(false)
		gl.MatrixMode(GL.MODELVIEW)
	end

	function gadget:DrawWorldPreUnit()
		gl.CallList(displayList)
	end
end